/**
 * Integrate Node-Sass into Webuilder
 *
 * Node-sass is a library that provides binding for Node.js to LibSass,
 * the C version of the popular stylesheet preprocessor, Sass.
 * It allows you to natively compile .scss files to css at incredible speed.
 * <https://github.com/sass/node-sass>
 * Requires nodejs. <https://nodejs.org/>
 *
 * @category  WeBuilder Plugin
 * @package   Node-Sass Integration
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2016
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.0
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     51   function GetOptions()
 *    102   function GetCompilerQueue(Sender)
 *    180   function CompileDocument()
 *    197   function OnSave(Sender)
 *    226   function CompileQueue()
 *    244   function NodeSass(queue)
 *    262   function DisplayError(error)
 *    303   function ShowSettings()
 *    879   function SelectFolder(Sender)
 *    893   function SelectFile(Sender)
 *    908   function SaveSettings()
 *    938   function OnExit()
 *    949   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 13
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Global vars.
 */
var settings = new TIniFile(Script.GetPath + "settings.ini");
var compilerQueue[0];

/**
 * Generate node-sass options from settings.
 *
 * @return string
 */
function GetOptions() {
    var linefeedTypes = ["cr", "crlf", "lf", "lfcr"], tmp, options = " -q";

    if (Script.Settings.TabsToSpaces) {
        options += " --indent-type space";
    }
    else {
        options += " --indent-type tab";
    }
    options += " --indent-width " + _t(Script.Settings.BlockIndent);
    options += " --linefeed " + linefeedTypes[settings.ReadInteger("Settings","LinefeedType", 1)];
    options += " --precision " + settings.ReadString("Settings", "Precision", "4");
    if (settings.ReadBool("Settings", "DebugEnable", false) == true) {
        options += " --source-comments";
    }
    if (settings.ReadBool("Settings", "SymlinkEnable", false) == true) {
        options += " --follow";
    }
    if (settings.ReadBool("Settings", "MapEnable", false) == false) {
        options += " --omit-source-map-url";
    }
    if (settings.ReadBool("Settings", "MapIncContentEnable", false) == true) {
        options += " --source-map-content";
    }
    if (settings.ReadBool("Settings", "MapIncUriEnable", false) == true) {
        options += " --source-map-embed";
   }

    tmp = Trim(settings.ReadString("Settings", "ImportPath", ""));
    if (tmp !="") {
        options += " --include-path \"" + tmp + "\"";
    }
    tmp = Trim(settings.ReadString("Settings", "CustomImportPath", ""));
    if (tmp !="") {
        options += " --functions \"" + tmp + "\"";
    }
    tmp = Trim(settings.ReadString("Settings", "CustomFuncPath", ""));
    if (tmp !="") {
        options += " --importer \"" + tmp + "\"";
    }

    return options;
}

/**
 * Create compiler queue.
 *
 * @param  object   Sender
 *
 * @return array
 */
function GetCompilerQueue(Sender) {
    var linefeedTypes = ["cr", "crlf", "lf", "lfcr"], tmp, queue[0], count = 0, outputStyle = "";
    var options = GetOptions();
    var source = Sender.Filename;
    var path = ExtractFilePath(source);
    var filename = ExtractFileName(source)
    var filenameNoSuffix = Replace(filename, ExtractFileExt(filename), "");
    var destination = "";

    if (settings.ReadBool("Settings", "CompactEnable", false) == true) {
        outputStyle = " --output-style compact";
        destination = filenameNoSuffix + settings.ReadString("Settings", "CompactSuffix", "-compact") + ".css";
        tmp = Trim(settings.ReadString("Settings", "CompactPath", ""));
        if (tmp != "") {
             destination = RegexReplace(tmp ,"\\\\$", "", false) + "\\" + destination;
        }
        if (settings.ReadBool("Settings", "MapEnable", false) == true) {
            outputStyle += " --source-map " + destination + ".map";
        }

        SetLength(queue,count+1);
        queue[count] = [path, outputStyle + options + " \"" + filename + "\" \"" + destination + "\""];
        count++;
    }

    if (settings.ReadBool("Settings", "CompressedEnable", false) == true) {
        outputStyle = " --output-style compressed";
        destination = filenameNoSuffix + settings.ReadString("Settings", "CompressedSuffix", "-compressed") + ".css";
        tmp = Trim(settings.ReadString("Settings", "CompressedPath", ""));
        if (tmp != "") {
             destination = RegexReplace(tmp ,"\\\\$", "", false) + "\\" + destination;
        }
        if (settings.ReadBool("Settings", "MapEnable", false) == true) {
            outputStyle += " --source-map " + destination + ".map";
        }
        SetLength(queue,count+1);
        queue[count] = [path, outputStyle + options + " \"" + filename + "\" \"" + destination + "\""];
        count++;
    }

    if (settings.ReadBool("Settings", "ExpandedEnable", false) == true) {
        outputStyle = " --output-style expanded";
        destination = filenameNoSuffix + settings.ReadString("Settings", "ExpandedSuffix", "-expanded") + ".css";
        tmp = Trim(settings.ReadString("Settings", "ExpandedPath", ""));
        if (tmp != "") {
             destination = RegexReplace(tmp ,"\\\\$", "", false) + "\\" + destination;
        }
        if (settings.ReadBool("Settings", "MapEnable", false) == true) {
            outputStyle += " --source-map " + destination + ".map";
        }
        SetLength(queue,count+1);
        queue[count] = [path, outputStyle + options + " \"" + filename + "\" \"" + destination + "\""];
        count++;
    }

    if (settings.ReadBool("Settings", "NestedEnable", false) == true) {
        outputStyle = " --output-style nested";
        destination = filenameNoSuffix + settings.ReadString("Settings", "NestedSuffix", "-nested") + ".css";
        tmp = Trim(settings.ReadString("Settings", "NestedPath", ""));
        if (tmp != "") {
             destination = RegexReplace(tmp ,"\\\\$", "", false) + "\\" + destination;
        }
        if (settings.ReadBool("Settings", "MapEnable", false) == true) {
            outputStyle += " --source-map " + destination + ".map";
        }
        SetLength(queue,count+1);
        queue[count] = [path, outputStyle + options + " \"" + filename + "\" \"" + destination + "\""];
        count++;
    }
    return queue;
}


/**
 * Manual trigger for compiling.
 *
 * @return void
 */
function CompileDocument() {
    if (Document.DocType == dtSass) {
        Document.Save(Document.FileName);
        OnSave(Document);
    }
    else {
        Alert("Error: Not a sass file!");
    }
}

/**
 * Event triggered when document has beeen saved.
 *
 * @param  object   Sender
 *
 * @return void
 */
function OnSave(Sender) {

    if (Sender.DocType == dtSass) {

        ExecuteCommand(Script.Path +  "node-sass-isinstalled.cmd", nodeSassIsInstalled);
        if  (nodeSassIsInstalled == 0) {
            Script.Message("Aborting: node-sass module NOT installed!");
            return;
        }

        if (Sender.Filename == "") {
            // Not a local file
            Script.Message("Aborting: node-sass can only compile files saved locally!");
            return;
        }

        compilerQueue = GetCompilerQueue(Sender);

        if (Length(compilerQueue) > 0) {
            Script.TimeOut(10,"CompileQueue");
        }
    }
}

/**
 * Loops through the queue of selected compiler outputs.
 *
 * @return void
 */
function CompileQueue() {
    var len = Length(compilerQueue);
    if (len > 0) {
        len--;
        var current = compilerQueue[len];
        SetLength(compilerQueue,len);
        NodeSass(current);
        Script.TimeOut(100,"CompileQueue");
    }
}

/**
 * Calls node-sass to compile document.
 *
 * @param  array   queue
 *
 * @return void
 */
function NodeSass(queue) {
    var error = "";
    ExecuteCommand(Script.Path + "node-sass.cmd \"" + queue[0] + "\" \"" + queue[1] + "\"", error);
    if (error !="") {
        SetLength(compilerQueue,0);
        DisplayError(error);
    }
}

/**
 * Displays error messages from node-sass in Webuilder message window.
 * node-sass returns error messages in json format which isn't supported by Webuilder plugin API.
 * So I simply parse each line until I find some matching keywords (message, line & column)
 *
 * @param  string   error
 *
 * @return void
 */
function DisplayError(error) {
    var SL = new TStringList, msg = "", line = "", column="", regex;
    SL.Text = error;

    for (var i=0;i<SL.Count;i++) {
        // Extract "message" part
        regex = "^\\s*\"message\":\\s\"(.*)\"";
        if (RegexMatch(SL[i], regex, true) != "") {
            msg = RegexReplace(RegexReplace(SL[i], regex, "$1", true), "\\\\\"", "\"", true);
            continue;
        }
        // Extract "column" part
        regex = "^\\s*\"column\":\\s(\\d+)";
        if (RegexMatch(SL[i], regex, true) != "") {
            column = _t(RegexReplace(SL[i], regex, "$1", true));
            continue;
        }
        // Extract "line" part
        regex = "^\\s*\"line\":\\s(\\d+)";
        if (RegexMatch(SL[i], regex, true) != "") {
            line = _t(RegexReplace(SL[i], regex, "$1", true));
            continue;
        }
    }

    if ((msg != "") && (line != "") && (column != "")) {
        Script.Message("Sass compile error: Line: " + line + ":" + column + ": " + msg);
    }
    else {
        // Could not parse the error, so we display everything.
        Script.Message(_t(error));
    }

    delete SL;
}

/**
 * Show Settings window.
 *
 * @return void
 */
function ShowSettings() {

    var mleft = 14,
        mtop = 14,
        vspace = 32,
        hdiv = 10,
        lspace = 4,
        bspace = 16,
        topPos = 0,
        leftPos = 0,
        gTopPos = 0,
        gLeftPos = 0;
        margin = 6;

    var modalForm = new TForm(WeBuilder);
    modalForm.ClientWidth = 700;
    modalForm.ClientHeight = 600;
    modalForm.Position = poScreenCenter;
    modalForm.BorderIcons = biSystemMenu; //remove maximize & minimize buttons
    modalForm.BorderStyle = bsSingle; //disable dialog resizing
    modalForm.Color = clBtnFace;
    modalForm.Font.Color = clWindowText;
    modalForm.Font.Height = -14;
    modalForm.Caption = "Node Sass Settings";

    var groupWidth = modalForm.ClientWidth - margin - margin - 8;
    gTopPos = margin;

    grpCompact = new TGroupBox(WeBuilder);
    grpCompact.Parent = modalForm;
    grpCompact.Caption 	= "Type: compact";
    grpCompact.Anchors = akLeft + akTop;
    grpCompact.Width = groupWidth;
    grpCompact.Left = margin;
    grpCompact.Top = gTopPos;

    topPos = mtop + mtop;
    leftPos = mleft;

    lblCompact1 =  new TLabel(WeBuilder);
    lblCompact1.Parent = grpCompact;
    lblCompact1.Anchors = akLeft + akTop;
    lblCompact1.ShowHint = true;
    lblCompact1.Hint = "Enable output of the compact type of CSS file";
    lblCompact1.Caption = "Enable output:";
    lblCompact1.SetBounds(leftPos, topPos + 3, 91, 17);

    leftPos += lblCompact1.Width + lspace;

    chkCompact = new TCheckBox(WeBuilder);
    chkCompact.Parent = grpCompact;
    chkCompact.ShowHint = true;
    chkCompact.Hint = "Enable output of the compact type of CSS file";
    chkCompact.Checked = settings.ReadBool("Settings", "CompactEnable", false);
    chkCompact.SetBounds(leftPos, topPos + 2, 22, 23);

    leftPos += chkCompact.Width + bspace;

    lblCompact2 =  new TLabel(WeBuilder);
    lblCompact2.Parent = grpCompact;
    lblCompact2.Anchors = akLeft + akTop;
    lblCompact2.ShowHint = true;
    lblCompact2.Hint = "Suffix of output filename";
    lblCompact2.Caption = "Filename suffix:";
    lblCompact2.SetBounds(leftPos, topPos + 3, 93, 17);

    leftPos += lblCompact2.Width + lspace;

    edCompact1 = new TEdit(WeBuilder);
    edCompact1.Parent = grpCompact;
    edCompact1.Anchors = akLeft  + akTop;
    edCompact1.ShowHint = true;
    edCompact1.Hint = "Suffix of output filename";
    edCompact1.Text = settings.ReadString("Settings", "CompactSuffix", "-compact");
    edCompact1.SetBounds( leftPos, topPos, 100, 25);

    leftPos += edCompact1.Width + bspace;

    lblCompact3 =  new TLabel(WeBuilder);
    lblCompact3.Parent = grpCompact;
    lblCompact3.Anchors = akLeft + akTop;
    lblCompact3.ShowHint = true;
    lblCompact3.Caption = "Output path:";
    lblCompact3.Hint = "Output path, relative to source file";
    lblCompact3.SetBounds(leftPos, topPos + 3, 81, 17);

    leftPos += lblCompact3.Width + lspace;

    edCompact2 = new TEdit(WeBuilder);
    edCompact2.Parent = grpCompact;
    edCompact2.Anchors = akLeft  + akTop;
    edCompact2.ShowHint = true;
    edCompact2.Hint = "Output path, relative to source file";
    edCompact2.Text = settings.ReadString("Settings", "CompactPath", "");
    edCompact2.SetBounds( leftPos, topPos, 225, 25);

    grpCompact.Height = topPos + mtop + mtop + 13;

    gTopPos += grpCompact.Height + margin;

    grpCompressed = new TGroupBox(WeBuilder);
    grpCompressed.Parent = modalForm;
    grpCompressed.Caption 	= "Type: compressed";
    grpCompressed.Anchors = akLeft + akTop;
    grpCompressed.Width = groupWidth;
    grpCompressed.Left = margin;
    grpCompressed.Top = gTopPos;

    topPos = mtop + mtop;
    leftPos = mleft;

    lblCompressed1 =  new TLabel(WeBuilder);
    lblCompressed1.Parent = grpCompressed;
    lblCompressed1.Anchors = akLeft + akTop;
    lblCompressed1.ShowHint = true;
    lblCompressed1.Hint = "Enable output of the compressed type of CSS file";
    lblCompressed1.Caption = "Enable output:";
    lblCompressed1.SetBounds(leftPos, topPos + 3, 91, 17);

    leftPos += lblCompressed1.Width + lspace;

    chkCompressed = new TCheckBox(WeBuilder);
    chkCompressed.Parent = grpCompressed;
    chkCompressed.ShowHint = true;
    chkCompressed.Hint = "Enable output of the compressed type of CSS file";
    chkCompressed.Checked = settings.ReadBool("Settings", "CompressedEnable", false);
    chkCompressed.SetBounds(leftPos, topPos + 2, 22, 23);

    leftPos += chkCompressed.Width + bspace;

    lblCompressed2 =  new TLabel(WeBuilder);
    lblCompressed2.Parent = grpCompressed;
    lblCompressed2.Anchors = akLeft + akTop;
    lblCompressed2.ShowHint = true;
    lblCompressed2.Hint = "Suffix of output filename";
    lblCompressed2.Caption = "Filename suffix:";
    lblCompressed2.SetBounds(leftPos, topPos + 3, 93, 17);

    leftPos += lblCompressed2.Width + lspace;

    edCompressed1 = new TEdit(WeBuilder);
    edCompressed1.Parent = grpCompressed;
    edCompressed1.Anchors = akLeft  + akTop;
    edCompressed1.ShowHint = true;
    edCompressed1.Hint = "Suffix of output filename";
    edCompressed1.Text = settings.ReadString("Settings", "CompressedSuffix", "-compressed");
    edCompressed1.SetBounds( leftPos, topPos, 100, 25);

    leftPos += edCompressed1.Width + bspace;

    lblCompressed3 =  new TLabel(WeBuilder);
    lblCompressed3.Parent = grpCompressed;
    lblCompressed3.Anchors = akLeft + akTop;
    lblCompressed3.ShowHint = true;
    lblCompressed3.Caption = "Output path:";
    lblCompressed3.Hint = "Output path, relative to source file";
    lblCompressed3.SetBounds(leftPos, topPos + 3, 81, 17);

    leftPos += lblCompressed3.Width + lspace;

    edCompressed2 = new TEdit(WeBuilder);
    edCompressed2.Parent = grpCompressed;
    edCompressed2.Anchors = akLeft  + akTop;
    edCompressed2.ShowHint = true;
    edCompressed2.Hint = "Output path, relative to source file";
    edCompressed2.Text = settings.ReadString("Settings", "CompressedPath", "");
    edCompressed2.SetBounds( leftPos, topPos, 225, 25);

    grpCompressed.Height = topPos + mtop + mtop + 13;

    gTopPos += grpCompressed.Height + margin;

    grpExpanded = new TGroupBox(WeBuilder);
    grpExpanded.Parent = modalForm;
    grpExpanded.Caption 	= "Type: expanded";
    grpExpanded.Anchors = akLeft + akTop;
    grpExpanded.Width = groupWidth;
    grpExpanded.Left = margin;
    grpExpanded.Top = gTopPos;

    topPos = mtop + mtop;
    leftPos = mleft;

    lblExpanded1 =  new TLabel(WeBuilder);
    lblExpanded1.Parent = grpExpanded;
    lblExpanded1.Anchors = akLeft + akTop;
    lblExpanded1.ShowHint = true;
    lblExpanded1.Hint = "Enable output of the expanded type of CSS file";
    lblExpanded1.Caption = "Enable output:";
    lblExpanded1.SetBounds(leftPos, topPos + 3, 91, 17);

    leftPos += lblExpanded1.Width + lspace;

    chkExpanded = new TCheckBox(WeBuilder);
    chkExpanded.Parent = grpExpanded;
    chkExpanded.ShowHint = true;
    chkExpanded.Hint = "Enable output of the expanded type of CSS file";
    chkExpanded.Checked = settings.ReadBool("Settings", "ExpandedEnable", false);
    chkExpanded.SetBounds(leftPos, topPos + 2, 22, 23);

    leftPos += chkExpanded.Width + bspace;

    lblExpanded2 =  new TLabel(WeBuilder);
    lblExpanded2.Parent = grpExpanded;
    lblExpanded2.Anchors = akLeft + akTop;
    lblExpanded2.ShowHint = true;
    lblExpanded2.Hint = "Suffix of output filename";
    lblExpanded2.Caption = "Filename suffix:";
    lblExpanded2.SetBounds(leftPos, topPos + 3, 93, 17);

    leftPos += lblExpanded2.Width + lspace;

    edExpanded1 = new TEdit(WeBuilder);
    edExpanded1.Parent = grpExpanded;
    edExpanded1.Anchors = akLeft  + akTop;
    edExpanded1.ShowHint = true;
    edExpanded1.Hint = "Suffix of output filename";
    edExpanded1.Text = settings.ReadString("Settings", "ExpandedSuffix", "-expanded");
    edExpanded1.SetBounds( leftPos, topPos, 100, 25);

    leftPos += edExpanded1.Width + bspace;

    lblExpanded3 =  new TLabel(WeBuilder);
    lblExpanded3.Parent = grpExpanded;
    lblExpanded3.Anchors = akLeft + akTop;
    lblExpanded3.ShowHint = true;
    lblExpanded3.Caption = "Output path:";
    lblExpanded3.Hint = "Output path, relative to source file";
    lblExpanded3.SetBounds(leftPos, topPos + 3, 81, 17);

    leftPos += lblExpanded3.Width + lspace;

    edExpanded2 = new TEdit(WeBuilder);
    edExpanded2.Parent = grpExpanded;
    edExpanded2.Anchors = akLeft  + akTop;
    edExpanded2.ShowHint = true;
    edExpanded2.Hint = "Output path, relative to source file";
    edExpanded2.Text = settings.ReadString("Settings", "ExpandedPath", "");
    edExpanded2.SetBounds( leftPos, topPos, 225, 25);

    grpExpanded.Height = topPos + mtop + mtop + 13;

    gTopPos += grpExpanded.Height + margin;

    grpNested = new TGroupBox(WeBuilder);
    grpNested.Parent = modalForm;
    grpNested.Caption 	= "Type: nested";
    grpNested.Anchors = akLeft + akTop;
    grpNested.Width = groupWidth;
    grpNested.Left = margin;
    grpNested.Top = gTopPos;

    topPos = mtop + mtop;
    leftPos = mleft;

    lblNested1 =  new TLabel(WeBuilder);
    lblNested1.Parent = grpNested;
    lblNested1.Anchors = akLeft + akTop;
    lblNested1.ShowHint = true;
    lblNested1.Hint = "Enable output of the nested type of CSS file";
    lblNested1.Caption = "Enable output:";
    lblNested1.SetBounds(leftPos, topPos + 3, 91, 17);

    leftPos += lblNested1.Width + lspace;

    chkNested = new TCheckBox(WeBuilder);
    chkNested.Parent = grpNested;
    chkNested.ShowHint = true;
    chkNested.Hint = "Enable output of the nested type of CSS file";
    chkNested.Checked = settings.ReadBool("Settings", "NestedEnable", false);
    chkNested.SetBounds(leftPos, topPos + 2, 22, 23);

    leftPos += chkNested.Width + bspace;

    lblNested2 =  new TLabel(WeBuilder);
    lblNested2.Parent = grpNested;
    lblNested2.Anchors = akLeft + akTop;
    lblNested2.ShowHint = true;
    lblNested2.Hint = "Suffix of output filename";
    lblNested2.Caption = "Filename suffix:";
    lblNested2.SetBounds(leftPos, topPos + 3, 93, 17);

    leftPos += lblNested2.Width + lspace;

    edNested1 = new TEdit(WeBuilder);
    edNested1.Parent = grpNested;
    edNested1.Anchors = akLeft  + akTop;
    edNested1.ShowHint = true;
    edNested1.Hint = "Suffix of output filename";
    edNested1.Text = settings.ReadString("Settings", "NestedSuffix", "-nested");
    edNested1.SetBounds( leftPos, topPos, 100, 25);

    leftPos += edNested1.Width + bspace;

    lblNested3 =  new TLabel(WeBuilder);
    lblNested3.Parent = grpNested;
    lblNested3.Anchors = akLeft + akTop;
    lblNested3.ShowHint = true;
    lblNested3.Caption = "Output path:";
    lblNested3.Hint = "Output path, relative to source file";
    lblNested3.SetBounds(leftPos, topPos + 3, 81, 17);

    leftPos += lblNested3.Width + lspace;

    edNested2 = new TEdit(WeBuilder);
    edNested2.Parent = grpNested;
    edNested2.Anchors = akLeft  + akTop;
    edNested2.ShowHint = true;
    edNested2.Hint = "Output path, relative to source file";
    edNested2.Text = settings.ReadString("Settings", "NestedPath", "");
    edNested2.SetBounds( leftPos, topPos, 225, 25);

    grpNested.Height = topPos + mtop + mtop + 13;

    gTopPos += grpNested.Height + margin;

    grpMaps = new TGroupBox(WeBuilder);
    grpMaps.Parent = modalForm;
    grpMaps.Caption = "Source Map";
    grpMaps.Anchors = akLeft + akTop;
    grpMaps.Width = groupWidth;
    grpMaps.Left = margin;
    grpMaps.Top = gTopPos;

    topPos = mtop + mtop;
    leftPos = mleft;

    lblMaps1 =  new TLabel(WeBuilder);
    lblMaps1.Parent = grpMaps;
    lblMaps1.Anchors = akLeft + akTop;
    lblMaps1.ShowHint = true;
    lblMaps1.Hint = "Enable creation of Source Map";
    lblMaps1.Caption = "Enable source map:";
    lblMaps1.SetBounds(leftPos, topPos + 3, 121, 17);

    leftPos += lblMaps1.Width + lspace;

    chkMaps1 = new TCheckBox(WeBuilder);
    chkMaps1.Parent = grpMaps;
    chkMaps1.ShowHint = true;
    chkMaps1.Hint = "Enable creation of Source Maps";
    chkMaps1.Checked = settings.ReadBool("Settings", "MapEnable", false);
    chkMaps1.SetBounds(leftPos, topPos + 2, 22, 23);

    leftPos += chkMaps1.Width + bspace;

    lblMaps2 =  new TLabel(WeBuilder);
    lblMaps2.Parent = grpMaps;
    lblMaps2.Anchors = akLeft + akTop;
    lblMaps2.ShowHint = true;
    lblMaps2.Hint = "Embed include contents in map";
    lblMaps2.Caption = "Embed include contents in map:";
    lblMaps2.SetBounds(leftPos, topPos + 3, 197, 17);

    leftPos += lblMaps2.Width + lspace;

    chkMaps2 = new TCheckBox(WeBuilder);
    chkMaps2.Parent = grpMaps;
    chkMaps2.ShowHint = true;
    chkMaps2.Hint = "Embed include contents in map";
    chkMaps2.Checked = settings.ReadBool("Settings", "MapIncContentEnable", false);
    chkMaps2.SetBounds(leftPos, topPos + 2, 22, 23);

    leftPos += chkMaps2.Width + bspace;

    lblMaps3 =  new TLabel(WeBuilder);
    lblMaps3.Parent = grpMaps;
    lblMaps3.Anchors = akLeft + akTop;
    lblMaps3.ShowHint = true;
    lblMaps3.Hint = "Embed sourceMappingUrl as data URI";
    lblMaps3.Caption = "Embed sourceMappingUrl as data URI:";
    lblMaps3.SetBounds(leftPos, topPos + 3, 234, 17);

    leftPos += lblMaps3.Width + lspace;

    chkMaps3 = new TCheckBox(WeBuilder);
    chkMaps3.Parent = grpMaps;
    chkMaps3.ShowHint = true;
    chkMaps3.Hint = "Embed sourceMappingUrl as data URI";
    chkMaps3.Checked = settings.ReadBool("Settings", "MapIncUriEnable", false);
    chkMaps3.SetBounds(leftPos, topPos + 2, 22, 23);

    grpMaps.Height = topPos + mtop + mtop + 13;
    gTopPos += grpMaps.Height + margin;

    grpMisc = new TGroupBox(WeBuilder);
    grpMisc.Parent = modalForm;
    grpMisc.Caption 	= "Misc";
    grpMisc.Anchors = akLeft + akTop;
    grpMisc.Width = groupWidth;
    grpMisc.Left = margin;
    grpMisc.Top = gTopPos;

    topPos = mtop + mtop;
    leftPos = mleft;

    lblMisc1 =  new TLabel(WeBuilder);
    lblMisc1.Parent = grpMisc;
    lblMisc1.Anchors = akLeft + akTop;
    lblMisc1.ShowHint = true;
    lblMisc1.Hint = "Linefeed style";
    lblMisc1.Caption = "Linefeed style:";
    lblMisc1.SetBounds(leftPos, topPos + 3, 87, 17);

    leftPos += lblMisc1.Width + lspace;

    cmbMisc1 = new TComboBox(WeBuilder);
    cmbMisc1.Parent = grpMisc;
    cmbMisc1.Anchors = akLeft + akTop;
    cmbMisc1.Style = csDropDownList;
    cmbMisc1.Items.Add("cr");
    cmbMisc1.Items.Add("crlf");
    cmbMisc1.Items.Add("lf");
    cmbMisc1.Items.Add("lfcr");
    cmbMisc1.ItemIndex = settings.ReadInteger("Settings","LinefeedType", 1);
    cmbMisc1.SetBounds(leftPos, topPos, 60, 17);

    leftPos += cmbMisc1.Width + bspace;

    lblMisc2 =  new TLabel(WeBuilder);
    lblMisc2.Parent = grpMisc;
    lblMisc2.Anchors = akLeft + akTop;
    lblMisc2.ShowHint = true;
    lblMisc2.Hint = "Decimal precision";
    lblMisc2.Caption = "Decimal precision:";
    lblMisc2.SetBounds(leftPos, topPos + 3, 109, 17);

    leftPos += lblMisc2.Width + lspace;

    edMisc1 = new TEdit(WeBuilder);
    edMisc1.Parent = grpMisc;
    edMisc1.Anchors = akLeft  + akTop;
    edMisc1.ShowHint = true;
    edMisc1.NumbersOnly = true;
    edMisc1.Hint = "Decimal precision";
    edMisc1.Text = settings.ReadString("Settings", "Precision", "4");
    edMisc1.SetBounds( leftPos, topPos, 50, 25);

    leftPos += edMisc1.Width + bspace;

    lblMisc3 =  new TLabel(WeBuilder);
    lblMisc3.Parent = grpMisc;
    lblMisc3.Anchors = akLeft + akTop;
    lblMisc3.ShowHint = true;
    lblMisc3.Hint = "Include debug info";
    lblMisc3.Caption = "Include debug info:";
    lblMisc3.SetBounds(leftPos, topPos + 3, 118, 17);

    leftPos += lblMisc3.Width + lspace;

    chkMisc1 = new TCheckBox(WeBuilder);
    chkMisc1.Parent = grpMisc;
    chkMisc1.ShowHint = true;
    chkMisc1.Hint = "Include debug info";
    chkMisc1.Checked = settings.ReadBool("Settings", "DebugEnable", false);
    chkMisc1.SetBounds(leftPos, topPos + 2, 22, 23);

    leftPos += chkMaps1.Width + bspace;

    lblMisc4 =  new TLabel(WeBuilder);
    lblMisc4.Parent = grpMisc;
    lblMisc4.Anchors = akLeft + akTop;
    lblMisc4.ShowHint = true;
    lblMisc4.Hint = "Follow symlinked directories";
    lblMisc4.Caption = "Follow symlinked dirs:";
    lblMisc4.SetBounds(leftPos, topPos + 3, 131, 17);

    leftPos += lblMisc4.Width + lspace;

    chkMisc2 = new TCheckBox(WeBuilder);
    chkMisc2.Parent = grpMisc;
    chkMisc2.ShowHint = true;
    chkMisc2.Hint = "Follow symlinked directories";
    chkMisc2.Checked = settings.ReadBool("Settings", "SymlinkEnable", false);
    chkMisc2.SetBounds(leftPos, topPos + 2, 22, 23);

    topPos += vspace;
    leftPos = mleft;

    lblMisc5 =  new TLabel(WeBuilder);
    lblMisc5.Parent = grpMisc;
    lblMisc5.Anchors = akLeft + akTop;
    lblMisc5.Alignment = taRightJustify;
    lblMisc5.ShowHint = true;
    lblMisc5.Hint = "Path to look for imported files";
    lblMisc5.Caption = "Path to look for imported files:";
    lblMisc5.SetBounds(leftPos, topPos + 3, 265, 17);

    leftPos += lblMisc5.Width + lspace;

    edMisc2 = new TEdit(WeBuilder);
    edMisc2.Parent = grpMisc;
    edMisc2.Anchors = akLeft + akTop;
    edMisc2.ShowHint = true;
    edMisc2.Hint = "Path to look for imported files";
    edMisc2.Text = settings.ReadString("Settings", "ImportPath", "");
    edMisc2.OnDblClick = "SelectFolder"
    edMisc2.SetBounds( leftPos, topPos, 390, 25);

    topPos += vspace;
    leftPos = mleft;

    lblMisc6 =  new TLabel(WeBuilder);
    lblMisc6.Parent = grpMisc;
    lblMisc6.Anchors = akLeft + akTop;
    lblMisc6.Alignment = taRightJustify;
    lblMisc6.ShowHint = true;
    lblMisc6.Hint = "Path to .js file containing custom importer";
    lblMisc6.Caption = "Path to .js file containing custom importer:";
    lblMisc6.SetBounds(leftPos, topPos + 3, 265, 17);

    leftPos += lblMisc6.Width + lspace;

    edMisc3 = new TEdit(WeBuilder);
    edMisc3.Parent = grpMisc;
    edMisc3.Anchors = akLeft  + akTop;
    edMisc3.ShowHint = true;
    edMisc3.Hint = "Path to .js file containing custom importer";
    edMisc3.Text = settings.ReadString("Settings", "CustomImportPath", "");
    edMisc3.OnDblClick = "SelectFile"
    edMisc3.SetBounds( leftPos, topPos, 390, 25);

    topPos += vspace;
    leftPos = mleft;

    lblMisc7 =  new TLabel(WeBuilder);
    lblMisc7.Parent = grpMisc;
    lblMisc7.Anchors = akLeft + akTop;
    lblMisc.Alignment = taRightJustify;
    lblMisc7.ShowHint = true;
    lblMisc7.Hint = "Path to .js file containing custom functions";
    lblMisc7.Caption = "Path to .js file containing custom functions:";
    lblMisc7.SetBounds(leftPos, topPos + 3, 265, 17);

    leftPos += lblMisc7.Width + lspace;

    edMisc4 = new TEdit(WeBuilder);
    edMisc4.Parent = grpMisc;
    edMisc4.Anchors = akLeft  + akTop;
    edMisc4.ShowHint = true;
    edMisc4.Hint = "Path to .js file containing custom functions";
    edMisc4.Text = settings.ReadString("Settings", "CustomFuncPath", "");
    edMisc4.OnDblClick = "SelectFile"
    edMisc4.SetBounds( leftPos, topPos, 390, 25);

    grpMisc.Height = topPos + mtop + mtop + 13;
    gTopPos += grpMisc.Height + margin;

    CancelButton = new TButton(WeBuilder);
    CancelButton.Parent = modalForm;
    CancelButton.Caption = "Cancel";
    CancelButton.ModalResult = mrCancel;
    CancelButton.SetBounds(modalForm.Width - margin - 125 - 98 - bspace, gTopPos, 98, 33);

    OKButton = new TButton(WeBuilder);
    OKButton.Parent = modalForm;
    OKButton.Caption = "OK";
    OKButton.ModalResult = mrOk;
    OKButton.SetBounds(modalForm.Width - margin - 125, gTopPos, 98, 33);

    var res = modalForm.Showmodal;
    if (res == mrOK) {
       SaveSettings();
    }

    delete modalForm;

}

/**
 * Bring up dialog to select folder.
 *
 * @param  object   Sender
 *
 * @return void
 */
function SelectFolder(Sender) {
    var path = ExtractFilePath(Sender.Text);
    if (BrowseForFolder(path, Sender.Hint, true)) {
        Sender.Text = path;
    }
}

/**
 * Bring up dialog to select file.
 *
 * @param  object   Sender
 *
 * @return void
 */
function SelectFile(Sender) {
    var opendialog = new TOpenDialog(WeBuilder);
    opendialog.InitialDir = Sender.Text;
    opendialog.Title = Sender.Hint;
    if (opendialog.Execute) {
        Sender.Text = opendialog.FileName;
    }
    delete opendialog;
}

/**
 * Save settings.
 *
 * @return void
 */
function SaveSettings() {
    settings.WriteBool("Settings", "CompactEnable", chkCompact.Checked);
    settings.WriteString("Settings", "CompactSuffix", edCompact1.Text);
    settings.WriteString("Settings", "CompactPath", edCompact2.Text);
    settings.WriteBool("Settings", "CompressedEnable", chkCompressed.Checked);
    settings.WriteString("Settings", "CompressedSuffix", edCompressed1.Text);
    settings.WriteString("Settings", "CompressedPath", edCompressed2.Text);
    settings.WriteBool("Settings", "ExpandedEnable", chkExpanded.Checked);
    settings.WriteString("Settings", "ExpandedSuffix", edExpanded1.Text);
    settings.WriteString("Settings", "ExpandedPath", edExpanded2.Text);
    settings.WriteBool("Settings", "NestedEnable", chkNested.Checked);
    settings.WriteString("Settings", "NestedSuffix", edNested1.Text);
    settings.WriteString("Settings", "NestedPath", edNested2.Text);
    settings.WriteBool("Settings", "MapEnable", chkMaps1.Checked);
    settings.WriteBool("Settings", "MapIncContentEnable", chkMaps2.Checked);
    settings.WriteBool("Settings", "MapIncUriEnable", chkMaps3.Checked);
    settings.WriteInteger("Settings","LinefeedType", cmbMisc1.ItemIndex);
    settings.WriteString("Settings", "Precision", edMisc1.Text);
    settings.WriteBool("Settings", "DebugEnable", chkMisc1.Checked);
    settings.WriteBool("Settings", "SymlinkEnable", chkMisc2.Checked);
    settings.WriteString("Settings", "ImportPath", edMisc2.Text);
    settings.WriteString("Settings", "CustomImportPath", edMisc3.Text);
    settings.WriteString("Settings", "CustomFuncPath", edMisc4.Text);
}

/**
 * Clean up if editor is closed.
 *
 * @return void
 */
function OnExit() {
    if (settings != null) {
        delete settings;
    }
}

/**
 * Show info when plugin is installed.
 *
 * @return void
 */
function OnInstalled() {
  Alert("Node Sass 1.0 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.ConnectSignal("exit", "OnExit");

var bmp = new TBitmap;
var icon = LoadFileToBitmap(Script.Path + "sass.bmp", bmp);

var config = Script.RegisterDocumentAction("", "Node Sass Settings", "", "ShowSettings");
Actions.SetIcon(config, bmp);


if (Script.ReadSetting("Autocompile when saving .scss file?", "1")=="1") {
    Script.ConnectSignal("document_after_save", "OnSave");
}
else {
    var compile = Script.RegisterDocumentAction("", "Node Sass Compiler", "", "CompileDocument");
     Actions.SetIcon(compile, bmp);
}

delete bmp;